function sot_dsc_read_orbit, filename, no = no, degree = degree

;+
; NAME:
;       SOT_DSC_READ_ORBIT.PRO
;
; PURPOSE:
;       Read orbital element data
;
; CALLING SEQUENCE:
;       oe = sot_dsc_read_orbit(filname, [no = no, /degree])
;
; INPUTS:
;       filename  -- file name of orbital element 
;                    
; KEYWORD PARAMETERS:
;       no -- number of a parameter set of orbital element 
;             [default: Select a newest parameter set of orbita element
;       degree -- angles are respsented in degree 
;
;
; OUTPUTS:
;       oe -- orbital element structure
; 
;
; MODIFICATION HISTORY:
;       05-Jul-2006 - Made by M.Kubo
;
;-



if keyword_set(degree) then dtor = 1 else dtor = !dpi/180D


oe = create_struct('NO', 0L, 'A', 0D, 'E', 0D, 'I', 0D, 'AN', 0D, $
                   'AP', 0D, 'MA', 0D, 'N', 0D, 'P', 0D, 'RA', 0D, $
                   'DEC', 0D, 'BET', 0D, 'GAM', 0D, 'dA', 0D, 'dE', 0D, $
                   'dI', 0D, 'dAN', 0D, 'dAP', 0D, 'dN', 0D, 'dP', 0D, $
                   'dRA', 0D, 'dDEC', 0D, 'dBET', 0D, 'dGAM', 0D, $
                   'time','', 'JD',0D)

;;;;;; Serch end of orbital element file
tmp = strarr(1)
num = 0
openr, unit, filename, /get_lun
repeat begin
    readf,unit,tmp
;    print,strmid(tmp,0,4)
    num = num + 1
endrep until (strmid(tmp,0,4) eq '====')
num = num -1
if n_elements(no) eq 0 then no = num - 1

free_lun, unit

;;;;;; read orbital element
tmp = dblarr(22,num)
oedata = dblarr(22)
openr, unit, filename, /get_lun
readf,unit,tmp
free_lun, unit
oedata = tmp[*,no]


;;;;;; make output data
oe.NO = oedata[0]
oe.A  = oedata[7]
oe.E  = oedata[8]
oe.I  = oedata[9]*dtor
oe.AN = oedata[10]*dtor
oe.AP = oedata[11]*dtor
oe.MA = oedata[12]
oe.N  = oedata[18]
oe.P  = oedata[20]/1440D

oe.dA = oedata[13]
oe.dE = oedata[14]
oe.dI = oedata[15]*dtor
oe.dAN = oedata[16]*dtor
oe.dAP = oedata[17]*dtor
oe.dN  = oedata[19]*dtor
oe.dP  = oedata[21]/1440D

oe.time = string(fix(oedata[1]),format='(i4.4)')+ '/' $ ;Year 
  + string(fix(oedata[2]),format='(i2.2)')+ '/' $ ;Month
  + string(fix(oedata[3]),format='(i2.2)')+ ' ' $ ;Day
  + string(fix(oedata[4]),format='(i2.2)')+ ':' $ ;Hour
  + string(fix(oedata[5]),format='(i2.2)')+ ':' $ ;Minute
  + string(fix(oedata[6]),format='(i2.2)')        ;Second


;;;;;; calculate RA and Dec of the Sun
sp = sot_dsc_sunpos(oe.time)

oe.RA  = sp.ra
oe.DEC = sp.dec
oe.dRA  = sp.dra
oe.dDEC = sp.ddec
oe.JD = sp.JD
 
;;; calculate beta and gamma angle
mv = 2*!dpi*oe.A/oe.P
oe.BET = asin(sin(oe.I)*sin(oe.AN)*cos(oe.DEC)*cos(oe.RA) - $
           sin(oe.I)*cos(oe.AN)*cos(oe.DEC)*sin(oe.RA) + $
           cos(oe.I)*sin(oe.DEC))
oe.GAM = atan(-cos(oe.I)*sin(oe.AN)*cos(oe.DEC)*cos(oe.RA) + $
           cos(oe.I)*cos(oe.AN)*cos(oe.DEC)*sin(oe.RA)  + $
           sin(oe.I)*sin(oe.DEC),                      $
           cos(oe.AN)*cos(oe.DEC)*cos(oe.RA) + $
           sin(oe.AN)*cos(oe.DEC)*sin(oe.AN))


;;; 1st order differential
oe.dBET = (1/cos(oe.BET))* $
  ((cos(oe.I)*cos(oe.AN - oe.RA) - sin(oe.I)*sin(oe.Dec))*oe.dI + $
   sin(oe.I)*cos(oe.DEC)*cos(oe.AN - oe.RA)*oe.dAN - $
   (sin(oe.I)*sin(oe.DEC)*sin(oe.AN) - cos(oe.I)*sin(oe.DEC))*oe.dDEC - $
   sin(oe.I)*cos(oe.DEC)*cos(oe.DEC)*cos(oe.AN - oe.RA)*oe.dRA)
oe.dGAM = (cos(oe.GAM))^2* $
  ((sin(oe.I)*(cos(oe.DEC))^2*sin(2*(oe.AN - oe.RA)) + $
    cos(oe.I)*cos(2*oe.DEC)*cos(oe.AN - oe.RA))*oe.dI  - $
   (cos(oe.I)*(cos(oe.DEC))^2 + $
    0.5*sin(oe.I)*sin(2*oe.DEC)*sin(oe.AN - oe.RA))*oe.dAN + $
   sin(oe.I)*cos(oe.AN - oe.RA)*oe.dDEC + $
   (cos(oe.I)*(cos(oe.DEC))^2 + $
    0.5*sin(oe.I)*sin(2*oe.DEC)*sin(oe.AN - oe.RA))*oe.dRA)


return,oe

end

