function sot_dsc_read_pointing, filename, $
                                degree = degree, delta_min = delta_min

;+
; NAME:
;       SOT_DSC_READ_POINTING.PRO
;
; PURPOSE:
;       Read an AOCS re-pointing file 
;
; CALLING SEQUENCE:
;       pt = sot_dsc_read_pointing(filename, [/degree, delta_min = delta_min]) 
;
; INPUTS:
;       filename -- filename of re-pointing file (re-point_yyyymmddpppp.txt)
;
; KEYWORD PARAMETERS:
;       degree -- angles are respsented in degree 
;       delta_min -- Delay time of "MDP_DSC_PAR2_UPDATE" to repointing time
;
; OUTPUTS:
;       pt -- Pointing information (structure)
; 
;
; MODIFICATION HISTORY:
;       05-Jul-2006 - Made by M.Kubo
;
;-

if n_elements(delta_min) eq 0 then delta_min = 1

EOF ='/* End of Tracking Curve Information */'
num = 0
tmp = ''
No = '00'

openr, unit, filename, /get_lun

; Search Re-pointing on the current table
repeat begin
    readf,unit,tmp       
    if strmid(tmp,1,14) eq 'Re-point Start' then begin
        if num eq 0 then begin
            rpt0 = No + ' ' + tmp
            num = 1
        endif else begin
            rpt0 = [rpt0, No + ' ' + tmp]
        endelse
    endif 
endrep until (strmid(tmp,0,16) eq 'AOCS Mem-Upload-')

; Search Re-pointing on new tables
repeat begin
    if strmid(tmp,0,16) eq 'AOCS Mem-Upload-' then No = strmid(tmp,16,2)
    readf,unit,tmp
    if strmid(tmp,1,14) eq 'Re-point Start' then begin
        if num eq 0 then begin
            rpt0 = No + ' ' + tmp
            num = 1
        endif else begin
            rpt0 = [rpt0, No + ' ' + tmp]
        endelse
    endif
endrep until (strmid(tmp,0,46) eq $
              '/* End of re-pointing and tracking schedule */')
    
repeat begin
    readf,unit,tmp
endrep until (strmid(tmp,0,55) eq $
      '/* AOCS Tracking Parameters before the memory upload */')

; Search Tracking Paramters on the current table
num = 0
repeat begin
    No = '00'
    readf,unit,tmp
    if strmid(tmp,0,5) eq 'Track' then begin
        if num eq 0 then begin
            trk0 = No + ' ' + tmp
            num = 1
        endif else begin
            trk0 = [trk0, No + ' ' + tmp]
        endelse
    endif
endrep until (strmid(tmp,0,57) eq $
              '/* New AOCS Tracking Parameters ' + $
              'after the memory upload*/' or $
              strmid(tmp,0,39) eq EOF)


; Search Tracking Paramters on new tables
if strmid(tmp,0,39) ne EOF then begin
    repeat begin
        if strmid(tmp,0,9) eq 'TR_MEM_ID' then No = strmid(tmp,10,2)
        if strmid(tmp,0,5) eq 'Track' then begin
            if num eq 0 then begin
                trk0 = No + ' ' + tmp
                num = 1
            endif else begin
                trk0 = [trk0, No + ' ' + tmp]
            endelse
        endif
        readf,unit,tmp
    endrep until (strmid(tmp,0,39) eq EOF)
endif
        
free_lun, unit


;Re-pointing parameters
rpt = replicate({mNo:0, time:'', tNo:0, offx:0D, offy:0D},n_elements(rpt0))
rpt.mNo = fix(strmid(rpt0,0,2))
rpt.time = strmid(rpt0,23,19)
rpt.tNo = fix(strmid(rpt0,46,1))
rpt.offx = double(strmid(rpt0,52,8))
rpt.offy = double(strmid(rpt0,61,8))

;Tracking parameters
trk = replicate({mNo:0, time:'', tNo:0, lon:0D, lat:0D, rot:0D},$
                n_elements(trk0))
trk.mNo = fix(strmid(trk0,0,2))
trk.tNo = fix(strmid(trk0,10,1))
trk.time = strmid(trk0,20,19)
trk.lon = double(strmid(trk0,42,6))
trk.lat = double(strmid(trk0,54,8))
trk.rot = double(strmid(trk0,65,10))


;Pointing infomation at MDP_DSC_PAR2_UPDATE
pt = replicate({time:'', lon:0D, lat:0D, trak:0},n_elements(rpt))
for i=0, n_elements(rpt)-1 do begin
    t0 = string(addtime(rpt[i].time, delta_min = delta_min), format='(I2.2)')
    pt[i].time = '20' +t0[6]+'/'+t0[5]+'/'+t0[4]+' '+ $
      t0[0]+':'+t0[1]+':'+t0[2] 

    if rpt[i].tNo eq 0 then begin
        hel = conv_a2h([-rpt[i].offy,-rpt[i].offx]*3600D,pt[i].time)
        pt[i].lon = hel[0]
        pt[i].lat = hel[1]
    endif else begin
        xx = where(trk.mNo eq rpt[i].mNO and trk.tNO eq rpt[i].tNo)
        dt = addtime(rpt[i].time,difference = trk[xx].time,/sec)+delta_min*60D 
        ang = conv_h2a([trk[i].lon + trk[i].rot*dt,trk[i].lat],pt[i].time)
        hel = conv_a2h([ang[0]-rpt[i].offy*3600D,$
                        ang[1]-rpt[i].offx*3600D],pt[i].time)
        pt[i].lon = hel[0]
        pt[i].lat = hel[1]
        pt[i].trak = 1
    endelse
endfor

;Degree -> Radiun
if 1-keyword_set(degree) then begin
    pt.lon = pt.lon*!dpi/180D
    pt.lat = pt.lat*!dpi/180D
endif

return, pt

end
