function sot_dsc_sunpos, time, dday = dday

;+
; NAME:
;       SOT_DSC_SUNPOS.PRO
;
; PURPOSE:
;       To compute the RA and Dec of the Sun at a given time, 
;         and compute their 1st order derivative during a dday
;
; CALLING SEQUENCE:
;       sp = sot_dsc_sunpos(time, [dday = dday]) 
;
; INPUTS:
;       time --  "yyyymmdd hh:mm:ss"
;
; KEYWORD PARAMETERS:
;       dday  -- Period to compute 1st derivation
;                [default: dday = 3 days]
;
; OUTPUTS:
;       sp -- sun position (structure)
; 
;
; MODIFICATION HISTORY:
;       05-Jul-2006 - Made by M.Kubo
;
;-


dtor = !dpi/180D
if n_elements(dday) eq 0 then dday = 3

;;;;;; Julian day
jd = JULDAY(strmid(time,5,2), $  ; Month
            strmid(time,8,2), $  ; Day
            strmid(time,0,4), $  ; Year
            strmid(time,11,2), $ ; Hour
            strmid(time,14,2), $ ; Minute
            strmid(time,17,2))   ; Second

;;;;;; caluculate Ra and Dec
sunpos,jd +0.25D*dindgen(dday*4),ra,dec


sp = create_struct('RA', 0D, 'DEC', 0D, 'dRA', 0D, 'dDEC', 0D, 'JD', 0D)
sp.RA = ra[0]*dtor
sp.dec = dec[0]*dtor

;;;;;; 1st order derivative of Ra and Dec
sp.dRA = average(deriv(ra,0.25D*dindgen(dday*4)))*dtor
sp.dDEC = average(deriv(dec,0.25D*dindgen(dday*4)))*dtor


sp.JD = jd

return, sp
end

